// Programmer: Seonwoo Kang
// Purpose: Maker faire exhibit (2024.11 Orlando)
// Date: 2024.9
// 
#include <LiquidCrystal_I2C.h>
#include <Wire.h>

int button_morse = 13;
int button_new_text = 11;
int buzzer = 10;

unsigned long time_not_push_button = 0; // don't use int. millis() return type is unsinged long.

LiquidCrystal_I2C lcd(0x27, 16, 2); // LiquidCrystal_I2C lcd(0x3F,16,2);
String morse_code = "";
String morse_text = "";

// list  of letters and numbers. https://en.wikipedia.org/wiki/ASCII
// String map_char_code[]={"A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z","0","1","2","3","4","5","6","7","8","9",
// ".",",","?","'","!","/","(",")","&",":",";","=","+","-","_","'","$","@","¿","¡"};
// String map_morse_code[]={".-","-...","-.-.","-..",".","..-.","--.","....","..",".---","-.-",".-..","--","-.","---",".--.","--.-",".-.","...","-","..-","...-",".--","-..-","-.--","--..","-----",".----","..---","...--","....-",".....","-....","--...","---..","----.",
// ".-.-.-","--..--","..--..",".----.","-.-.--","-..-.","-.--.","-.--.-",".-...","---...","-.-.-.","-...-",".-.-.","-....-","..--.-",".-..-.","...-..-",".--.-.","..-.-","--...-"};
String map_char_code[]={"A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z","0","1","2","3","4","5","6","7","8","9", ".",",","?","'","!","/","(",")"};
String map_morse_code[]={".-","-...","-.-.","-..",".","..-.","--.","....","..",".---","-.-",".-..","--","-.","---",".--.","--.-",".-.","...","-","..-","...-",".--","-..-","-.--","--..",
"-----",".----","..---","...--","....-",".....","-....","--...","---..","----.", ".-.-.-","--..--","..--..",".----.","-.-.--","-..-.","-.--.","-.--.-"};

void  setup() {
	// put your setup code here, to run once:
	pinMode(button_morse,  INPUT);
	pinMode(button_new_text, INPUT);
	pinMode(buzzer, OUTPUT);
	Serial.begin(9600); 

	lcd.init();       // Inicialize communication with  the display
	lcd.backlight();  // Turn the display lights on
	lcd.clear();      // Clean the display
	lcd.setCursor(0,0);
	lcd.print("Hi. Morse world!");
	delay(3000);
	lcd.clear();      // Clean the display
}

void display_lcd(String code, String text)
{
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(code);    
  lcd.setCursor(0, 1);
  lcd.print(text);    
}

String complate_morse_code(String morse_code)
{
  int length = sizeof(map_char_code) / sizeof(String);
  for(int i = 0; i < length; i++) // compare the list of morse_codes with  the list of morse
  {
    if(map_morse_code[i] == morse_code) // if the morse_code  made is equal to the respective morse code, print the morse_code that have the same  index
    {
      return map_char_code[i];
    }
  }    
  return " ";
}

void loop() {
  int button_morse_state = digitalRead(button_morse);
  int button_new_state = digitalRead(button_new_text);
  unsigned long time_start  = millis();   // time_start restart before press the button
  while (button_morse_state){   // while the button is pressed the time is count
    button_morse_state = digitalRead(button_morse);
    tone(buzzer, 800);
  }
  noTone(buzzer);

  if(button_new_state)
  {
    tone(buzzer, 800);
    delay(200);
    noTone(buzzer);
     
    Serial.println(morse_text);
    morse_code = morse_text = "";
    display_lcd(morse_code, morse_text);
    delay(500);  
    return; 
  }

  int time_pressed_button = millis() - time_start; // finalize the time and store in time_pressed_button
  if(time_pressed_button < 5)
  {
    int morse_code_length = morse_code.length();
    if(morse_code_length > 0)
    {
      if(time_not_push_button <= 0) 
      {
        // Serial.println("1 | " + morse_code + " | " + String(time_pressed_button) + " | " + String(time_not_push_button));  // ebug
        time_not_push_button = millis();
      }

      if((millis() - time_not_push_button) > 1500) // after 1.5 second, user don't push the button, complate morse code.
      {
        // Serial.println("2 | " + morse_code + " | " + String(time_pressed_button) + " | " + String(time_not_push_button)); // debug
        time_not_push_button = 0;
        String morse_char = complate_morse_code(morse_code);
        morse_code = "";

        morse_text = morse_text + morse_char;
        display_lcd(morse_code, morse_text);
        delay(500);              
      }
    }
  }
  else if(time_pressed_button  < 200) // if the button is pressed for less than 200 miliseconds  and more than 5 miliseconds, so the morse_code is "."
  {
    if(morse_code.length() == 0)
      lcd.clear();
    morse_code = morse_code + ".";
    display_lcd(morse_code, morse_text);
    time_not_push_button = 0;
  }
  else if(time_pressed_button < 1000) // if the button is pressed  for more than 200 seconds, so the morse_code is "-" 
  {
    if(morse_code.length() == 0)
      lcd.clear();
    morse_code = morse_code + "-";
    display_lcd(morse_code, morse_text);
    time_not_push_button = 0;
  }
}
